/* tslint:disable */
/* eslint-disable */
export const memory: WebAssembly.Memory;
export function get_odoh_config(): number;
export function decrypt_query(a: number, b: number): number;
export function create_response(a: number, b: number): number;
export function __wbindgen_malloc(a: number): number;
